@@ -13,20 +13,21 @@ use {
         pyalloc::PythonMemoryAllocator,
     },
     once_cell::sync::Lazy,
-    pyo3::{
-        exceptions::PyRuntimeError, ffi as pyffi, prelude::*, types::PyDict, PyTypeInfo,
-        ToBorrowedObject,
-    },
-    python_oxidized_importer::{
+    oxidized_importer::{
         install_path_hook, remove_external_importers, replace_meta_path_importers, ImporterState,
         OxidizedFinder, PyInit_oxidized_importer, PythonResourcesState, OXIDIZED_IMPORTER_NAME,
         OXIDIZED_IMPORTER_NAME_STR,
     },
+    pyo3::{
+        exceptions::PyRuntimeError, ffi as pyffi, prelude::*, types::PyDict, PyTypeInfo,
+        ToBorrowedObject,
+    },
     python_packaging::interpreter::{MultiprocessingStartMethod, TerminfoResolution},
     std::{
         collections::BTreeSet,
         env, fs,
         io::Write,
+        os::raw::c_char,
         path::{Path, PathBuf},
     },
 };
@@ -46,13 +47,49 @@ static GLOBAL_INTERPRETER_GUARD: Lazy<std::sync::Mutex<()>> =
 /// This type and its various functionality is a glorified wrapper around the
 /// Python C API. But there's a lot of added functionality on top of what the C
 /// API provides.
+///
+/// # Usage
+///
+/// Construct instances via [MainPythonInterpreter::new()]. This will acquire
+/// a global lock and initialize the main Python interpreter in the current
+/// process.
+///
+/// Python code can then be executed in the interpreter in any number of
+/// different ways.
+///
+/// If you want to run whatever was configured to run via the
+/// [OxidizedPythonInterpreterConfig] used to construct the instance, call
+/// [MainPythonInterpreter::run()] or [MainPythonInterpreter::py_runmain()].
+/// The former will honor "multiprocessing worker" and is necessary for
+/// `multiprocessing` to work. [MainPythonInterpreter::py_runmain()] bypasses
+/// multiprocessing mode checks.
+///
+/// If you want to execute arbitrary Python code or want to run Rust code
+/// with the GIL held, call [MainPythonInterpreter::with_gil()]. The provided
+/// function will be provided a [pyo3::Python], which represents a handle on
+/// the Python interpreter. This function is just a wrapper around
+/// [pyo3::Python::with_gil()]. But since the function holds a reference to
+/// self, it prevents [MainPythonInterpreter] from being dropped prematurely.
+///
+/// # Safety
+///
+/// Dropping a [MainPythonInterpreter] instance will call `Py_FinalizeEx()` to
+/// finalize the Python interpreter and prevent it from running any more Python
+/// code.
+///
+/// If a Python C API is called after interpreter finalization, a segfault can
+/// occur.
+///
+/// If you use pyo3 APIs like [Python::with_gil()] directly, you may
+/// inadvertently attempt to operate on a finalized interpreter. Therefore
+/// it is recommended to always go through a method on an [MainPythonInterpreter]
+/// instance in order to interact with the Python interpreter.
 pub struct MainPythonInterpreter<'interpreter, 'resources: 'interpreter> {
     // It is possible to have a use-after-free if config is dropped before the
     // interpreter is finalized/dropped.
     config: ResolvedOxidizedPythonInterpreterConfig<'resources>,
     interpreter_guard: Option<std::sync::MutexGuard<'interpreter, ()>>,
     pub(crate) allocator: Option<PythonMemoryAllocator>,
-    gil: Option<GILGuard>,
     /// File to write containing list of modules when the interpreter finalizes.
     write_modules_path: Option<PathBuf>,
 }
@@ -82,7 +119,6 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
             config,
             interpreter_guard: None,
             allocator: None,
-            gil: None,
             write_modules_path: None,
         };
 
@@ -102,15 +138,13 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
     /// If called more than once, the function is a no-op from the perspective
     /// of interpreter initialization.
     ///
-    /// Returns a Python instance which has the GIL acquired.
+    /// The GIL is not held after the interpreter is initialized.
     fn init(&mut self) -> Result<(), NewInterpreterError> {
         assert!(self.interpreter_guard.is_none());
         self.interpreter_guard = Some(GLOBAL_INTERPRETER_GUARD.lock().map_err(|_| {
             NewInterpreterError::Simple("unable to acquire global interpreter guard")
         })?);
 
-        let origin_string = self.config.origin().display().to_string();
-
         if let Some(tcl_library) = &self.config.tcl_library {
             std::env::set_var("TCL_LIBRARY", tcl_library);
         }
@@ -178,6 +212,9 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
             ));
         }
 
+        // The GIL is held.
+        debug_assert_eq!(unsafe { pyffi::PyGILState_Check() }, 1);
+
         // At this point, the core of Python is initialized.
         // importlib._bootstrap has been loaded. But not
         // importlib._bootstrap_external. This is where we work our magic to
@@ -186,14 +223,13 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
         let oxidized_finder_loaded =
             unsafe { Python::with_gil_unchecked(|py| self.inject_oxidized_importer(py))? };
 
+        // The GIL is still held after calling into PyO3.
+        debug_assert_eq!(unsafe { pyffi::PyGILState_Check() }, 1);
+
         // Now proceed with the Python main initialization. This will initialize
         // importlib. And if the custom importlib bytecode was registered above,
         // our extension module will get imported and initialized.
         let status = unsafe { pyffi::_Py_InitializeMain() };
-
-        let gil = Python::acquire_gil();
-        let py = gil.python();
-
         if unsafe { pyffi::PyStatus_Exception(status) } != 0 {
             return Err(NewInterpreterError::new_from_pystatus(
                 &status,
@@ -201,6 +237,90 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
             ));
         }
 
+        // The GIL is held after finishing initialization.
+        debug_assert_eq!(unsafe { pyffi::PyGILState_Check() }, 1);
+
+        // We release the GIL so we can have pyo3's GIL handling take over from
+        // an "empty" state. This mirrors what pyo3's prepare_freethreaded_python() does.
+        unsafe {
+            pyffi::PyEval_SaveThread();
+        }
+
+        self.write_modules_path =
+            self.with_gil(|py| self.init_post_main(py, oxidized_finder_loaded))?;
+
+        debug_assert_eq!(unsafe { pyffi::PyGILState_Check() }, 0);
+
+        Ok(())
+    }
+
+    /// Inject OxidizedFinder into Python's importing mechanism.
+    ///
+    /// This function is meant to be called as part of multi-phase interpreter initialization
+    /// after `Py_InitializeFromConfig()` but before `_Py_InitializeMain()`. Calling it
+    /// any other time may result in errors.
+    ///
+    /// Returns whether an `OxidizedFinder` was injected into the interpreter.
+    fn inject_oxidized_importer(&self, py: Python) -> Result<bool, NewInterpreterError> {
+        if !self.config.oxidized_importer {
+            return Ok(false);
+        }
+
+        let resources_state = Box::new(PythonResourcesState::try_from(&self.config)?);
+
+        let oxidized_importer = py.import(OXIDIZED_IMPORTER_NAME_STR).map_err(|err| {
+            NewInterpreterError::new_from_pyerr(py, err, "import of oxidized importer module")
+        })?;
+
+        let cb = |importer_state: &mut ImporterState| match self.config.multiprocessing_start_method
+        {
+            MultiprocessingStartMethod::None => {}
+            MultiprocessingStartMethod::Fork
+            | MultiprocessingStartMethod::ForkServer
+            | MultiprocessingStartMethod::Spawn => {
+                importer_state.set_multiprocessing_set_start_method(Some(
+                    self.config.multiprocessing_start_method.to_string(),
+                ));
+            }
+            MultiprocessingStartMethod::Auto => {
+                // Windows uses "spawn" because "fork" isn't available.
+                // Everywhere else uses "fork." The default on macOS is "spawn." This
+                // is due to https://bugs.python.org/issue33725, which only affects
+                // Python framework builds. Our assumption is we aren't using a Python
+                // framework, so "spawn" is safe.
+                let method = if cfg!(target_family = "windows") {
+                    "spawn"
+                } else {
+                    "fork"
+                };
+
+                importer_state.set_multiprocessing_set_start_method(Some(method.to_string()));
+            }
+        };
+
+        // Ownership of the resources state is transferred into the importer, where the Box
+        // is summarily leaked. However, the importer tracks a pointer to the resources state
+        // and will constitute the struct for dropping when it itself is dropped. We could
+        // potentially encounter a use-after-free if the importer is used after self.config
+        // is dropped. However, that would require self to be dropped. And if self is dropped,
+        // there should no longer be a Python interpreter around. So it follows that the
+        // importer state cannot be dropped after self.
+
+        replace_meta_path_importers(py, oxidized_importer, resources_state, Some(cb)).map_err(
+            |err| {
+                NewInterpreterError::new_from_pyerr(py, err, "initialization of oxidized importer")
+            },
+        )?;
+
+        Ok(true)
+    }
+
+    /// Performs interpreter configuration after main interpreter initialization.
+    fn init_post_main(
+        &self,
+        py: Python,
+        oxidized_finder_loaded: bool,
+    ) -> Result<Option<PathBuf>, NewInterpreterError> {
         let sys_module = py
             .import("sys")
             .map_err(|e| NewInterpreterError::new_from_pyerr(py, e, "obtaining sys module"))?;
@@ -272,7 +392,7 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
             let argvb = b"argvb\0";
 
             let res = args.with_borrowed_ptr(py, |args_ptr| unsafe {
-                pyffi::PySys_SetObject(argvb.as_ptr() as *const i8, args_ptr)
+                pyffi::PySys_SetObject(argvb.as_ptr() as *const c_char, args_ptr)
             });
 
             match res {
@@ -286,7 +406,7 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
         let oxidized = b"oxidized\0";
 
         let res = true.into_py(py).with_borrowed_ptr(py, |py_true| unsafe {
-            pyffi::PySys_SetObject(oxidized.as_ptr() as *const i8, py_true)
+            pyffi::PySys_SetObject(oxidized.as_ptr() as *const c_char, py_true)
         });
 
         match res {
@@ -298,7 +418,7 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
             let frozen = b"frozen\0";
 
             match true.into_py(py).with_borrowed_ptr(py, |py_true| unsafe {
-                pyffi::PySys_SetObject(frozen.as_ptr() as *const i8, py_true)
+                pyffi::PySys_SetObject(frozen.as_ptr() as *const c_char, py_true)
             }) {
                 0 => (),
                 _ => return Err(NewInterpreterError::Simple("unable to set sys.frozen")),
@@ -307,24 +427,24 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
 
         if self.config.sys_meipass {
             let meipass = b"_MEIPASS\0";
-            let value = origin_string.to_object(py);
+            let value = self.config.origin().display().to_string().to_object(py);
 
             match value.with_borrowed_ptr(py, |py_value| unsafe {
-                pyffi::PySys_SetObject(meipass.as_ptr() as *const i8, py_value)
+                pyffi::PySys_SetObject(meipass.as_ptr() as *const c_char, py_value)
             }) {
                 0 => (),
                 _ => return Err(NewInterpreterError::Simple("unable to set sys._MEIPASS")),
             }
         }
 
-        if let Some(key) = &self.config.write_modules_directory_env {
+        let write_modules_path = if let Some(key) = &self.config.write_modules_directory_env {
             if let Ok(path) = std::env::var(key) {
                 let path = PathBuf::from(path);
 
                 std::fs::create_dir_all(&path).map_err(|e| {
                     NewInterpreterError::Dynamic(format!(
                         "error creating directory for loaded modules files: {}",
-                        e.to_string()
+                        e
                     ))
                 })?;
 
@@ -346,87 +466,27 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
                     })?
                     .to_string();
 
-                self.write_modules_path = Some(path.join(format!("modules-{}", uuid_str)));
-            }
-        }
-
-        Ok(())
-    }
-
-    /// Inject OxidizedFinder into Python's importing mechanism.
-    ///
-    /// This function is meant to be called as part of multi-phase interpreter initialization
-    /// after `Py_InitializeFromConfig()` but before `_Py_InitializeMain()`. Calling it
-    /// any other time may result in errors.
-    ///
-    /// Returns whether an `OxidizedFinder` was injected into the interpreter.
-    fn inject_oxidized_importer(&self, py: Python) -> Result<bool, NewInterpreterError> {
-        if !self.config.oxidized_importer {
-            return Ok(false);
-        }
-
-        let resources_state = Box::new(PythonResourcesState::try_from(&self.config)?);
-
-        let oxidized_importer = py.import(OXIDIZED_IMPORTER_NAME_STR).map_err(|err| {
-            NewInterpreterError::new_from_pyerr(py, err, "import of oxidized importer module")
-        })?;
-
-        let cb = |importer_state: &mut ImporterState| match self.config.multiprocessing_start_method
-        {
-            MultiprocessingStartMethod::None => {}
-            MultiprocessingStartMethod::Fork
-            | MultiprocessingStartMethod::ForkServer
-            | MultiprocessingStartMethod::Spawn => {
-                importer_state.set_multiprocessing_set_start_method(Some(
-                    self.config.multiprocessing_start_method.to_string(),
-                ));
-            }
-            MultiprocessingStartMethod::Auto => {
-                // Windows uses "spawn" because "fork" isn't available.
-                // Everywhere else uses "fork." The default on macOS is "spawn." This
-                // is due to https://bugs.python.org/issue33725, which only affects
-                // Python framework builds. Our assumption is we aren't using a Python
-                // framework, so "spawn" is safe.
-                let method = if cfg!(target_family = "windows") {
-                    "spawn"
-                } else {
-                    "fork"
-                };
-
-                importer_state.set_multiprocessing_set_start_method(Some(method.to_string()));
+                Some(path.join(format!("modules-{}", uuid_str)))
+            } else {
+                None
             }
+        } else {
+            None
         };
 
-        // Ownership of the resources state is transferred into the importer, where the Box
-        // is summarily leaked. However, the importer tracks a pointer to the resources state
-        // and will constitute the struct for dropping when it itself is dropped. We could
-        // potentially encounter a use-after-free if the importer is used after self.config
-        // is dropped. However, that would require self to be dropped. And if self is dropped,
-        // there should no longer be a Python interpreter around. So it follows that the
-        // importer state cannot be dropped after self.
-
-        replace_meta_path_importers(py, oxidized_importer, resources_state, Some(cb)).map_err(
-            |err| {
-                NewInterpreterError::new_from_pyerr(py, err, "initialization of oxidized importer")
-            },
-        )?;
-
-        Ok(true)
+        Ok(write_modules_path)
     }
 
-    /// Ensure the Python GIL is released.
-    pub fn release_gil(&mut self) {
-        self.gil = None;
-    }
-
-    /// Ensure the Python GIL is acquired, returning a handle on the interpreter.
+    /// Proxy for [Python::with_gil()].
     ///
-    /// The returned value has a lifetime of the `MainPythonInterpreter`
-    /// instance. This is because `MainPythonInterpreter.drop()` finalizes
-    /// the interpreter. The borrow checker should refuse to compile code
-    /// where the returned `Python` outlives `self`.
-    pub fn acquire_gil(&mut self) -> Python<'_> {
-        self.gil.get_or_insert_with(Python::acquire_gil).python()
+    /// This allows running Python code via the PyO3 Rust APIs. Alternatively,
+    /// this can be used to run code when the Python GIL is held.
+    #[inline]
+    pub fn with_gil<F, R>(&self, f: F) -> R
+    where
+        F: for<'py> FnOnce(Python<'py>) -> R,
+    {
+        Python::with_gil(f)
     }
 
     /// Runs `Py_RunMain()` and finalizes the interpreter.
@@ -440,7 +500,13 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
     /// the evaluation result, consider calling a function on the interpreter handle
     /// that executes code.
     pub fn py_runmain(self) -> i32 {
-        unsafe { pyffi::Py_RunMain() }
+        unsafe {
+            // GIL must be acquired before calling Py_RunMain(). And Py_RunMain()
+            // finalizes the interpreter. So we don't need to release the GIL
+            // afterwards.
+            pyffi::PyGILState_Ensure();
+            pyffi::Py_RunMain()
+        }
     }
 
     /// Run in "multiprocessing worker" mode.
@@ -448,7 +514,7 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
     /// This should be called when `sys.argv[1] == "--multiprocessing-fork"`. It
     /// will parse arguments for the worker from `sys.argv` and call into the
     /// `multiprocessing` module to perform work.
-    pub fn run_multiprocessing(&mut self) -> PyResult<i32> {
+    pub fn run_multiprocessing(&self) -> PyResult<i32> {
         // This code effectively reimplements multiprocessing.spawn.freeze_support(),
         // except entirely in the Rust domain. This function effectively verifies
         // `sys.argv[1] == "--multiprocessing-fork"` then parsed key=value arguments
@@ -462,41 +528,42 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
             panic!("run_multiprocessing() called prematurely; sys.argv does not indicate multiprocessing mode");
         }
 
-        let py = self.acquire_gil();
-        let kwargs = PyDict::new(py);
+        self.with_gil(|py| {
+            let kwargs = PyDict::new(py);
 
-        for arg in argv.iter().skip(2) {
-            let arg = arg.to_string_lossy();
+            for arg in argv.iter().skip(2) {
+                let arg = arg.to_string_lossy();
 
-            let mut parts = arg.splitn(2, '=');
+                let mut parts = arg.splitn(2, '=');
 
-            let key = parts
-                .next()
-                .ok_or_else(|| PyRuntimeError::new_err("invalid multiprocessing argument"))?;
-            let value = parts
-                .next()
-                .ok_or_else(|| PyRuntimeError::new_err("invalid multiprocessing argument"))?;
+                let key = parts
+                    .next()
+                    .ok_or_else(|| PyRuntimeError::new_err("invalid multiprocessing argument"))?;
+                let value = parts
+                    .next()
+                    .ok_or_else(|| PyRuntimeError::new_err("invalid multiprocessing argument"))?;
 
-            let value = if value == "None" {
-                py.None()
-            } else {
-                let v = value.parse::<isize>().map_err(|e| {
-                    PyRuntimeError::new_err(format!(
-                        "unable to convert multiprocessing argument to integer: {}",
-                        e
-                    ))
-                })?;
+                let value = if value == "None" {
+                    py.None()
+                } else {
+                    let v = value.parse::<isize>().map_err(|e| {
+                        PyRuntimeError::new_err(format!(
+                            "unable to convert multiprocessing argument to integer: {}",
+                            e
+                        ))
+                    })?;
 
-                v.into_py(py)
-            };
+                    v.into_py(py)
+                };
 
-            kwargs.set_item(key, value)?;
-        }
+                kwargs.set_item(key, value)?;
+            }
 
-        let spawn_module = py.import("multiprocessing.spawn")?;
-        spawn_module.getattr("spawn_main")?.call0()?;
+            let spawn_module = py.import("multiprocessing.spawn")?;
+            spawn_module.getattr("spawn_main")?.call1((kwargs,))?;
 
-        Ok(0)
+            Ok(0)
+        })
     }
 
     /// Whether the Python interpreter is in "multiprocessing worker" mode.
@@ -517,13 +584,15 @@ impl<'interpreter, 'resources> MainPythonInterpreter<'interpreter, 'resources> {
     /// and dispatch to multiprocessing accordingly.
     ///
     /// Otherwise, this delegates to [Self::py_runmain].
-    pub fn run(mut self) -> i32 {
+    pub fn run(self) -> i32 {
         if self.config.multiprocessing_auto_dispatch && self.is_multiprocessing() {
             match self.run_multiprocessing() {
                 Ok(code) => code,
                 Err(e) => {
-                    let py = self.acquire_gil();
-                    e.print(py);
+                    self.with_gil(|py| {
+                        e.print(py);
+                    });
+
                     1
                 }
             }
@@ -645,17 +714,26 @@ fn write_modules_to_path(py: Python, path: &Path) -> Result<(), &'static str> {
 
 impl<'interpreter, 'resources> Drop for MainPythonInterpreter<'interpreter, 'resources> {
     fn drop(&mut self) {
-        if let Some(path) = self.write_modules_path.clone() {
-            if let Err(msg) = write_modules_to_path(self.acquire_gil(), &path) {
-                eprintln!("error writing modules file: {}", msg);
+        // Interpreter may have been finalized already. Possibly through our invocation
+        // of Py_RunMain(). Possibly something out-of-band beyond our control. We don't
+        // muck with the interpreter after finalization because this will likely result
+        // in a segfault.
+        if unsafe { pyffi::Py_IsInitialized() } == 0 {
+            return;
+        }
+
+        if let Some(path) = self.write_modules_path.as_ref() {
+            match self.with_gil(|py| write_modules_to_path(py, path)) {
+                Ok(_) => {}
+                Err(msg) => {
+                    eprintln!("error writing modules file: {}", msg);
+                }
             }
         }
 
-        // This will drop the GILGuard if it is defined. This needs to be called
-        // before we finalize the interpreter, otherwise GILGuard's drop may try
-        // to DECREF objects after they are freed by Py_FinalizeEx().
-        self.gil = None;
-
-        let _ = unsafe { pyffi::Py_FinalizeEx() };
+        unsafe {
+            pyffi::PyGILState_Ensure();
+            pyffi::Py_FinalizeEx();
+        }
     }
 }
